# Copyright(c) 2003-2006 Association of Universities for Research in Astronomy, Inc.

procedure girmfringe(inimages,fringe)

# Scale and subtract a fringe frame from a GMOS gireduced image
# 
# Version Jul 30, 2003  BM  created
#         Aug 27, 2003  KL  IRAF2.12 - new parameters
#                             imstat: nclip, lsigma, hsigma, cache
#         Dec 08, 2003 PLG  Changed scale = 1.0 as default
#         Feb 22, 2004 BM   Changeed default scale back to 0.0, needed or OLDP 
#                           and exposure time scaling is needed in general

string  inimages    {"",prompt="Input MEF images"}                      # OLDP-1-input-primary-single-prefix=f
string  outimages   {"",prompt="Output MEF images"}                     # OLDP-1-output
string  outpref     {"f",prompt="Prefix for output images"}             # OLDP-4
string  fringe      {"",prompt="Fringe frame"}                          # OLDP-1-input
bool    fl_statscale    {no,prompt="Scale by statistics rather than exposure time?"} # OLDP-2
string  statsec     {"[SCI,2][*,*]",prompt="Extension and region for satistics"} # OLDP-2
real    scale       {0.0,prompt="Override auto-scaling if not 0.0"}     # OLDP-2
string  logfile     {"",prompt="Log file"}                              # OLDP-1
bool    verbose     {yes,prompt="Verbose output?"}                      # OLDP-4
int     status      {0,prompt="Exit status (0=good)"}                   # OLDP-4
struct  *scanfile   {"",prompt="Internal use only"}                     # OLDP-4

begin

    string  l_inimages, l_outimages, l_fringe, l_statsec
    string  l_logfile, l_outpref
    real    l_scale
    int     l_status
    bool    l_verbose, l_fl_statsc

    string  filelist, img, suf, infile[500], outfile[500]
    real    expfringe, expfile, ratio, med, stda, stdb
    int     ii, kk, nbad, nsciext, nf, len, idx
    struct  sdate

    # Query parameters
    l_inimages=inimages ; l_outimages=outimages ; l_outpref=outpref
    l_fringe=fringe ; l_logfile=logfile ; l_statsec=statsec
    l_scale=scale
    l_verbose=verbose ; l_fl_statsc=fl_statscale

    status = 0

    # Keep imgets parameters from changing by outside world
    cache ("imgets","hedit","gemlogname")

    # Define the name of the logfile
    gemlogname (logpar=l_logfile, package="gmos")
    l_logfile = gemlogname.logname

    date | scan(sdate)
    printlog ("---------------------------------------------------------------\
        -----------------", l_logfile, verbose=l_verbose)
    printlog ("GIRMFRINGE -- "//sdate, l_logfile, verbose=l_verbose)
    printlog (" ", l_logfile, verbose=l_verbose)
    printlog ("inimages = "//l_inimages, l_logfile, verbose=l_verbose)
    printlog ("outimages = "//l_outimages, l_logfile, verbose=l_verbose)
    printlog ("outpref = "//l_outpref, l_logfile, verbose=l_verbose)
    printlog ("fringe = "//l_fringe, l_logfile, verbose=l_verbose)
    printlog ("fl_statscale = "//l_fl_statsc, l_logfile, verbose=l_verbose)
    printlog ("statsec = "//l_statsec, l_logfile, verbose=l_verbose)
    printlog ("scale = "//l_scale, l_logfile, verbose=l_verbose)
    printlog (" ", l_logfile, verbose=l_verbose)

    # Define temporary files
    filelist = mktemp("tmpfilelist")

    #check that a fringe frame is given
    if (l_fringe == "" || l_fringe == " "){
        printlog ("ERROR - GIRMFRINGE: fringe frame not specified",
            l_logfile, yes)
        goto error
    }

    # Check that the fringe frame exists
    if (!imaccess(l_fringe)) {  
        printlog ("ERROR - GIRMFRINGE: Fringe frame "//l_fringe//" does \
            not exist.", l_logfile, yes)
        goto error
    }
    imgets (l_fringe//"[0]","NSCIEXT", >& "dev$null")
    nsciext = int(imgets.value)

    #check that there are input files
    if (l_inimages == "" || l_inimages == " ") {
        printlog ("ERROR - GIRMFRINGE: inimages files not specified",
            l_logfile, yes)
        goto error
    }

    #check statistics section
    if (l_fl_statsc && l_scale != 0.0 && (l_statsec=="" || l_statsec==" ")) {
        printlog ("ERROR - GIRMFRINGE: statistics section not given",
            l_logfile, yes)
        goto error
    }

    # check existence of input list file
    idx = stridx("@",l_inimages)
    if (idx != 0) {
        len = strlen(l_inimages)
        if (!access(substr(l_inimages,(idx+1),len))) {
            printlog ("ERROR - GIRMFRINGE: "//substr(l_inimages,(idx+1),len)//\
                " does not exist.", l_logfile, yes)
            goto error
        }
    }
    files (l_inimages,sort-, > filelist)

    # Check that all images in the list exist and are MEF
    nbad = 0
    kk = 0
    scanfile = filelist
    while (fscan(scanfile, img) != EOF) {
        gimverify (img)
        if (gimverify.status>0) {
            nbad=nbad+1
        } else {
            kk = kk+1
            if (kk > 500) {
                printlog ("ERROR - GIRMFRINGE: Number of images exceeds array \
                    limit.", l_logfile, l_verbose)
                goto error
            }
            # name w/o suffix
            infile[kk] = gimverify.outname//".fits"
            imgets (infile[kk]//"[0]","NSCIEXT", >& "dev$null")
            if (nsciext != int(imgets.value)) {
                printlog ("ERROR - GIRMFRINGE: "//infile[kk]//" has a \
                    different number of extensions than the fringe frame.",
                    l_logfile, yes)
                goto error
            }
        }
    }
    nf = kk
    scanfile = ""
    delete (filelist, verify-, >& "dev$null")

    # Exit if problems found
    if (nbad > 0) {
        printlog ("ERROR - GIRMFRINGE: "//nbad//" image(s) either do not \
            exist or are not MEF files.", l_logfile, yes)
        goto error
    }

    # Check or create output file names
    if (l_outimages != "" && l_outimages != " "){
        # list file?
        idx = stridx("@",l_outimages)
        if (idx != 0) {
            len = strlen(l_outimages)
            if (!access(substr(l_outimages,(idx+1),len))) {
                printlog ("ERROR - GIRMFRINGE: "//\
                    substr(l_outimages,(idx+1),len)//" does not exist.",
                    l_logfile, yes)
                goto error
            }
        }
        files (l_outimages,sort-, > filelist)
        scanfile = filelist
        nbad = 0
        kk = 0
        while(fscan(scanfile, img) != EOF) {
            if (imaccess(img)) {
                nbad = nbad+1
            }
            kk = kk+1
            if (kk > 500) {
                printlog ("ERROR - GIRMFRINGE: Number of output images \
                    exceeds array limit.", l_logfile, l_verbose)
                goto error
            }
            #put suffix on output image
            len = strlen(img)
            suf = substr(img,(len-4),len)
            if (suf != ".fits") {
                outfile[kk] = img//".fits"
            } else {
                outfile[kk] = img
            }
        }
        scanfile = ""
        delete (filelist,verify-, >& "dev$null")
        if (nbad > 0) {
            printlog ("ERROR - GIRMFRINGE: "//nbad//" output images already \
                exist", l_logfile, yes)
            goto error
        }
        if (kk != nf) {
            printlog ("ERROR - GIRMFRINGE: number of input and output images \
                not the same.", l_logfile, yes)
            goto error
        }
    } else {
        if (l_outpref=="" || l_outpref==" ") {
            printlog ("ERROR - GIRMFRINGE: output prefix not given",
                l_logfile, yes)
            goto error
        }
        for (ii=1; ii<=nf; ii+=1) {
            outfile[ii] = l_outpref//infile[ii]
        }
    }


    # Main loop
    # Get exposure time for fringe frame
    hselect (l_fringe//"[0]","EXPTIME","yes") | scan(expfringe)
    if (l_scale == 0.0 && l_fl_statsc) {
        imstat(l_fringe//l_statsec, fields="midpt,stddev", lower=INDEF,
            upper=INDEF, nclip=0, lsigma=INDEF, usigma=INDEF, binwidth=0.1,
            format-, cache-) | scan (med, stda)
    }
    ratio = l_scale
    for (ii=1; ii<=nf; ii+=1) {
        #Exposure time
        if (l_scale == 0.0) {
            if (l_fl_statsc) {
                imstat (infile[ii]//l_statsec, fields="midpt,stddev",
                    lower=0.01, upper=INDEF, nclip=0, lsigma=INDEF,
                    usigma=INDEF, binwidth=0.1, format-, cache-) | \
                    scan (med, stdb)
                imstat (infile[ii]//l_statsec, fields="midpt,stddev",
                    lower=(med-3.*stdb), upper=(med+2.5*stdb), nclip=0,
                    lsigma=INDEF, usigma=INDEF, binwidth=0.1, format-, 
                    cache-) | scan (med, stdb)
                ratio = stdb/stda
            } else {
                hselect (infile[ii]//"[0]", "EXPTIME", "yes") | scan (expfile)
                ratio = expfile/expfringe
            }
        }
        printlog (infile[ii]//" "//outfile[ii]//" ratio = "//ratio,
            l_logfile, l_verbose)
        copy (infile[ii], outfile[ii], verb-)
        for (kk=1; kk<=nsciext; kk+=1) {
            imexpr ("a-b*c", outfile[ii]//"[SCI,"//kk//",overwrite+]",\
                infile[ii]//"[SCI,"//kk//"]", ratio,\
                l_fringe//"[SCI,"//kk//"]", verbose-)
        }
        
        gemdate ()
        nhedit (outfile[ii]//"[0]", "GIRMFRIN", gemdate.outdate,
            "UT Time stamp form GIRMFRINGE", comfile="NULL", after="",
            before="", update+, add+, addonly-, delete-, verify-, show-)
        nhedit (outfile[ii]//"[0]", "GEM-TLM", gemdate.outdate,
            "UT Last modification with GEMINI", comfile="NULL", after="",
            before="", update+, add+, addonly-, delete-, verify-, show-)
    }
    goto clean

error:
    status = 1
    goto clean

clean:
    delete (filelist, verify-, >& "dev$null")
    # close log file
    printlog (" ", l_logfile, l_verbose)
    printlog ("GIRMFRINGE done", l_logfile, l_verbose)
    printlog ("-----------------------------------------------------------\
        ---------------------", l_logfile, l_verbose )

end
