# Copyright(c) 2003-2005 Association of Universities for Research in Astronomy , Inc.
procedure mdfplot(intable)

# Plot MDF
#
# Version Apr 23, 2003  IJ  First draft, minimal checking of input
#         Jul 15, 2003  IJ  updated for 2003B conventions, handle GS automatically
#         Aug 12, 2003  IJ  updated for the 0/1 semester convention
#	  Oct 13, 2003  KL  moved to gmos$mostools
#         Dec 10, 2003  IJ  backwards compatable, make it work for DD/SV programs

char      intable   {prompt="MDF FITS file be plotted"}
bool      fl_ps     {yes,prompt="Make PostScript on disk"}
char      barcode   {"default",prompt="Barcode for mask"}
bool      fl_inter  {yes,prompt="Examine mask plot interactively"}
bool      fl_over   {yes,prompt="Overwrite existing PostScript on disk"}
char      logfile   {"",prompt="Logfile"}
bool      verbose   {yes,prompt="Verbose?"}
int       status    {0,prompt="Exit status (0=good)"}
char      *scanfile {"",prompt="For internal use only"}

begin

char l_intable, l_logfile, l_barcode
bool l_verbose, l_fl_ps, l_fl_over, l_fl_inter

char l_rootname, l_postscript, tmpgki, s_empty, tmpigi, l_nrootname, l_mdfid
struct l_struct
int l_ii, l_rows, l_istr
real xpos, ypos, tilt, xsize, ysize, y1, y2, x11, x12, x21, x22
bool l_fl_old

l_intable=intable ; l_logfile=logfile ; l_verbose=verbose ; l_fl_ps=fl_ps
l_barcode=barcode ; l_fl_over=fl_over ; l_fl_inter=fl_inter
status=0 

tmpgki=mktemp("tmpgki")
tmpigi=mktemp("tmpigi")

cache("gimverify","imgets","parkey","tinfo","fparse")

# Define the name of the logfile
s_empty=""; print(l_logfile) | scan(s_empty); l_logfile=s_empty
if (l_logfile == "") {
  l_logfile = mostools.logfile
  if (l_logfile == "") {
     l_logfile = "gmos.log"
     printlog("WARNING - MDFPLOT: both mdfplot.logfile and \
      mostools.logfile are empty.",l_logfile,l_verbose)
     printlog("                Using default file gmos.log.",l_logfile,l_verbose)
  }
}

# Write to the logfile
date | scan(l_struct)
printlog("-----------------------------------------------------------------\
-----------",l_logfile,l_verbose)
printlog("MDFPLOT -- "//l_struct,l_logfile,l_verbose)
printlog("",l_logfile,l_verbose)

if(!l_fl_ps && !l_fl_inter) {
  printlog("ERROR - MDFPLOT: Using fl_ps=no and fl_inter=no produces no output",l_logfile,yes)
  goto crash
}

# Make sure input table has .fits suffix
gimverify(l_intable) ; l_intable=gimverify.outname//".fits"
if(!access(l_intable)) {
  printlog("ERROR - MDFPLOT: Input table "//l_intable//" not found",l_logfile,yes)
  goto crash
}

fparse(l_intable) ; l_rootname=fparse.root

if(access(l_rootname//".ps") && l_fl_over==no) {
   printlog("ERROR - MDFPLOT: Output PostScript file exists",l_logfile,yes)
   goto crash
} else 
  delete(l_rootname//".ps",ver-, >>& "dev$null")

# Make a rootname without - to be used for the barcode seaching
# First get the mask running number, which in new MDFs is always 2 digits
l_fl_old=no
l_mdfid=substr(l_rootname,strlen(l_rootname)-1,strlen(l_rootname))
if(substr(l_mdfid,1,1)=="-") {
  l_fl_old=yes
  l_mdfid=substr(l_mdfid,2,2)
}
print(l_rootname) | translit("STDIN","-","",delete+,collapse-) | scan(l_nrootname)
if(l_fl_old) 
  l_nrootname=substr(l_nrootname,1,strlen(l_nrootname)-1)//"-0"//l_mdfid
else
  l_nrootname=substr(l_nrootname,1,strlen(l_nrootname)-2)//"-"//l_mdfid

# Need to know the location of the one remaining "-"
l_istr=stridx("-",l_nrootname)

# Decide on the barcode
if(l_barcode=="default") {
 s_empty=""
 print(l_nrootname) | match("A","STDIN") | scan(s_empty)
 if(s_empty!="")
    l_barcode="10"
 else
    l_barcode="11"

 s_empty=""
 print(l_nrootname) | match("Q","STDIN") | scan(s_empty)
 if(s_empty!="") {
    l_barcode=l_barcode//"0"
    s_empty=substr(l_nrootname,9,l_istr-1)
    l_barcode=l_barcode//"000"+int(s_empty)
    print(l_nrootname) | tokens("STDIN",newlines-) | \
     fields("STDIN","1",lines="2") | scan(s_empty)
    s_empty=substr(s_empty,2,strlen(s_empty))
    l_barcode=l_barcode//"00"+int(s_empty)

 } else {
 print(l_nrootname) | match("D","STDIN") | scan(s_empty)
  if(s_empty!="") {
    l_barcode=l_barcode//"9"
    s_empty=substr(l_nrootname,10,l_istr-1)
    l_barcode=l_barcode//"000"+int(s_empty)
    print(l_nrootname) | tokens("STDIN",newlines-) | \
     fields("STDIN","1",lines="2") | scan(s_empty)
    s_empty=substr(s_empty,2,strlen(s_empty))
    l_barcode=l_barcode//"00"+int(s_empty)

  } else {
   print(l_nrootname) | match("V","STDIN") | scan(s_empty)
  if(s_empty!="") {
    l_barcode=l_barcode//"8"
    s_empty=substr(l_nrootname,10,l_istr-1)
    l_barcode=l_barcode//"000"+int(s_empty)
    print(l_nrootname) | tokens("STDIN",newlines-) | \
     fields("STDIN","1",lines="2") | scan(s_empty)
    s_empty=substr(s_empty,2,strlen(s_empty))
    l_barcode=l_barcode//"00"+int(s_empty)

  } else {
   print(l_nrootname) | match("C","STDIN") | scan(s_empty)
  if(s_empty!="") {
    l_barcode=l_barcode//"1"
    s_empty=substr(l_nrootname,9,l_istr-1)
    l_barcode=l_barcode//"000"+int(s_empty)
    print(l_nrootname) | tokens("STDIN",newlines-) | \
     fields("STDIN","1",lines="2") | scan(s_empty)
    s_empty=substr(s_empty,2,strlen(s_empty))
    l_barcode=l_barcode//"00"+int(s_empty)
  } else {
   printlog("WARNING - MDFPLOT: Cannot assign barcode",l_logfile,verbose+)
   l_barcode="None"
  }

  }
  }

 }

} 

# Make the plot
mgograph(l_intable,"slitpos_mx","slitpos_my",pointe=0.1,marker="cross",
wx1=-120,wx2=120,wy1=-120,wy2=120,xlab="X mask",ylab="Y mask",
title=l_rootname//" "//l_barcode,postitle="topleft",labelexp=1.5, >>G tmpgki)

# Make igi command file that draws the slits with the correct sizee and tilt
tinfo(l_intable,ttout-, >>& "dev$null")
l_rows=tinfo.nrows
for(l_ii=1;l_ii<=l_rows;l_ii+=1) {
 tprint(l_intable,showr-,row=str(l_ii),showh-,option="plain",prdata+,prparam-,
  col="slitpos_mx,slitpos_my,slittilt_m,slitsize_mx,slitsize_my",pwidth=156) | \
  scan(xpos,ypos,tilt,xsize,ysize)

y1=ypos-ysize/2. ;  y2=ypos+ysize/2.
x11=xpos-xsize/2.+ysize/2*(-cos(tilt/180.*3.1416))
x12=xpos+xsize/2.+ysize/2*(-cos(tilt/180.*3.1416))
x21=xpos-xsize/2.-ysize/2*(-cos(tilt/180.*3.1416))
x22=xpos+xsize/2.-ysize/2*(-cos(tilt/180.*3.1416))

if(l_ii==1) 
  head("mostools$mdfplot.igi",nlines=3, > tmpigi)

print("relo "//str(x11)//" "//str(y1), >> tmpigi)
print("draw "//str(x12)//" "//str(y1)//" ; draw "//str(x22)//" "//str(y2),>> tmpigi)
print("draw "//str(x21)//" "//str(y2)//" ; draw "//str(x11)//" "//str(y1),>> tmpigi)
}

type(tmpigi) | igi(append+,debug-,device="stdgraph", >>G tmpgki)

# Draw the frame
type("mostools$mdfplot.igi") | igi(append+,debug-,device="stdgraph", >>G tmpgki)

if(l_fl_inter)
  gkimos(tmpgki,nx=1,ny=1)

if(l_fl_ps) {
  gkimos(tmpgki,nx=1,ny=1,dev="psfsq")
  gflush
  sleep 3
  rename("iraf.ps",l_rootname//".ps", field="all")
}

printlog("MDFPLOT - MDF "//l_rootname//" Barcode "//l_barcode,l_logfile,l_verbose)
if(l_fl_ps)
  printlog("MDFPLOT - PostScript file "//l_rootname//".ps",l_logfile,l_verbose)

goto clean

crash:
status=1
goto theend

clean:
status=0

theend:
delete(tmpgki//","//tmpigi,ver-, >>& "dev$null")
printlog("-----------------------------------------------------------------\
-----------",l_logfile,l_verbose)

end
