# Copyright(c) 2005 Association of Universities for Research in Astronomy, Inc.
#
# GNIRS IFU Science data reduction script
#
# Observation UT dates:  April 9, 2004
#
# This data was taken during the commissioning of the GNIRS IFU mode.
#
# Required packages:
#   gemini
#   gnirs
#

# Location of the raw data
set rawdir=/export/data/gemini_testdata/gnirs_testdata/ifu/

# Location of the reduced calibration data
set calibdir=rawdir$IFUexample-CAL/

# Load required tasks/packages
gemini
gemtools
gnirs

#--------------setup for gnirs-----------
# set default parameters
# use gnirs defaults except where specified on command lines below
unlearn gemini
unlearn gemtools
unlearn gnirs

# set display
set stdimage=imt1024

# set logfile
gnirs.logfile="GS-ENG20040409.log"

# set raw path
nsprepare.rawpath="rawdir$"

# load gnirs header keywords
nsheaders gnirs

####################################################
##### Basic reduction of the science exposures #####
####################################################

# shiftimage is set to one of the GCAL flat images that
# were used to create the flat field.  The MDF shift to
# apply will
# be taken from that image
gemlist range="162-171" root="S20040409S" > sci.lis
nsprepare @sci.lis shiftimage="calibdir$nS20040409S0161" \
    bpm="calibdir$rnS20040409S0161_flat_bpm.pl"
nsreduce n@sci.lis fl_nscut+ fl_nsappwave+ fl_dark- fl_sky+ fl_flat+ \
    skyimages="" skyrange=INDEF flatimage="calibdir$rnS20040409S0161_sflat"

###################################################
##### Combine spectra taken at same pointings #####
###################################################

#  Note: 5 images, 3 nod positions
#        Nod position #1 : S20040409S0162       (single => no need to stack)
#        Nod position #2 : S20040409S0165 to 166
#        Nod position #3 : S20040409S0169 to 170

imdelete ("rnS20040409S0165_stack,rnS20040409S0166_stack",
    verify-, >& "dev$null")
delete ("stack.lis", verify-, >& "dev$null")
gemlist range="165-166,169-170" root="rnS20040409S" > stack.lis
nsstack @stack.lis tolerance=0.1 rejtype="none" lthreshold=-100 hthreshold=150

###################################################
##### Resample the spectra onto a linear grid #####
###################################################

# Need local copy of frames used for calibration
imdelete wrnS20040409S0155,snS20040409S0161_pin ver- >& dev$null
copy calibdir$wrnS20040409S0155.fits .
copy calibdir$snS20040409S0161_pin.fits .

# If calibration database in another directory (e.g. without write permission),
# then need to copy database
#
#if (access("GS-ENG20040409-CAL_database"))
#   delete GS-ENG20040409-CAL_database/* ver- >& dev$null
#else
#   !mkdir GS-ENG20040409-CAL_database
#copy calibdir$GS-ENG20040409-CAL_database/* GS-ENG20040409-CAL_database/

imdelete ("trnS20040409S0162,trnS20040409S0165_stack,trnS20040409S0166_stack",
    verify-, >& "dev$null")
nstransform rnS20040409S0162,rnS20040409S0165_stack,rnS20040409S0166_stack \
    lamp=wrnS20040409S0155.fits sdist=snS20040409S0161_pin \
    database=GS-ENG20040409-CAL_database/

##################################################
##### Stack the 2D spectra into 3D datacubes #####
##################################################

nfcube trnS20040409S0162,trnS20040409S0165_stack,trnS20040409S0166_stack \
    process="rotate"



